@@ -1,21 +1,37 @@
 package WWW::Curl;
 
 use strict;
-use vars qw(@ISA $VERSION);
-use DynaLoader;
+use warnings;
+use XSLoader;
 
+our $VERSION;
 BEGIN {
-    $VERSION = '4.00';
-    @ISA     = qw(DynaLoader);
-    __PACKAGE__->bootstrap;
+	$VERSION = '4.19_9901';
+	XSLoader::load(__PACKAGE__, $VERSION);
+}
+
+END {
+    _global_cleanup();
+}
+
+our $AUTOLOAD;
+sub AUTOLOAD {
+    (my $constname = $AUTOLOAD) =~ s/.*:://;
+    die "&WWW::Curl::constant not defined" if $constname eq 'constant';
+    my ($error, $val) = constant($constname);
+    if ($error) {
+        my (undef,$file,$line) = caller;
+        die "$error at $file line $line.\n";
+    }
+    no strict 'refs';
+    *$AUTOLOAD = sub { $val };
+    goto &$AUTOLOAD;
 }
 
 1;
 
 __END__
 
-=cut 
-
 =head1 NAME
 
 WWW::Curl - Perl extension interface for libcurl
@@ -48,19 +64,14 @@ Here is a small snippet of making a request with WWW::Curl::Easy.
 	use warnings;
 	use WWW::Curl::Easy;
 
-	# Setting the options
-	my $curl = new WWW::Curl::Easy;
+	my $curl = WWW::Curl::Easy->new;
 	
-	# Redirecting the default STDOUT target for header contents, to
-	# an anonymous temporary file
-	open(my $fileh, '+>', undef);
-	$curl->setopt(CURLOPT_HEADERDATA,$fileh);
-
 	$curl->setopt(CURLOPT_HEADER,1);
 	$curl->setopt(CURLOPT_URL, 'http://example.com');
+
+	# A filehandle, reference to a scalar or reference to a typeglob can be used here.
 	my $response_body;
-	open (my $fileb, ">", \$response_body);
-	$curl->setopt(CURLOPT_WRITEDATA,$fileb);
+	$curl->setopt(CURLOPT_WRITEDATA,\$response_body);
 
 	# Starts the actual request
 	my $retcode = $curl->perform;
@@ -72,9 +83,11 @@ Here is a small snippet of making a request with WWW::Curl::Easy.
 		# judge result and next action based on $response_code
 		print("Received response: $response_body\n");
 	} else {
-		print("An error happened: ".$curl->strerror($retcode)." ($retcode)\n");
+		# Error code, type of error, error message
+		print("An error happened: $retcode ".$curl->strerror($retcode)." ".$curl->errbuf."\n");
 	}
 
+See L<curl_easy_setopt(3)> for details of C<setopt()>.
 
 =head1 WWW::Curl::Multi
 
@@ -84,7 +97,7 @@ Here is a small snippet of making a request with WWW::Curl::Easy.
 	use WWW::Curl::Multi;
 
 	my %easy;
-	my $curl = WWW::Curl::Easy->new:
+	my $curl = WWW::Curl::Easy->new;
 	my $curl_id = '13'; # This should be a handle unique id.
 	$easy{$curl_id} = $curl;
 	my $active_handles = 0;
@@ -99,20 +112,27 @@ Here is a small snippet of making a request with WWW::Curl::Easy.
 	$curlm->add_handle($curl);
 	$active_handles++;
 
-	while (my $active_transfers = $curlm->perform) {
-	if ($active_transfers != $active_handles) {
-		while (my ($id,$return_value) = $curlm->info_read) {
-			if ($id) {
-				$active_handles--;
-				my $actual_easy_handle = $easy{$id};
-				# do the usual result/error checking routine here
-				...
-				# letting the curl handle get garbage collected, or we leak memory.
-				delete $easy{$id};
+	while ($active_handles) {
+		my $timeout = $curlm->timeout();
+		if ( $timeout > 1 ) {
+			my ($rv, $wv, $ev) = $curlm->fdset_vec;
+			select $rv, $wv, $ev, $timeout / 1000;
+		}
+
+		my $active_transfers = $curlm->perform;
+		if ($active_transfers != $active_handles) {
+			while (my ($id,$return_value) = $curlm->info_read) {
+				if ($id) {
+					$active_handles--;
+					my $actual_easy_handle = $easy{$id};
+					# do the usual result/error checking routine here
+					...
+					# letting the curl handle get garbage collected, or we leak memory.
+					delete $easy{$id};
+				}
 			}
 		}
 	}
-	}
 
 This interface is different than what the C API does. $curlm->perform is non-blocking and performs
 requests in parallel. The method does a little work and then returns control, therefor it has to be called
@@ -168,6 +188,26 @@ This is how you enable sharing for a specific WWW::Curl::Easy handle:
 	$curl->setopt(CURLOPT_SHARE, $curlsh)
 		Attach share object to WWW::Curl::Easy instance
 
+=head1 WWW::Curl::Form
+
+    use WWW::Curl::Form;
+    my $curlf = WWW::Curl::Form->new;
+    $curlf->formaddfile($filename, 'attachment', "multipart/form-data");
+    $curlf->formadd("FIELDNAME", "VALUE");
+
+    $curl->setopt(CURLOPT_HTTPPOST, $curlf);
+
+Its usable methods are:
+
+    $curlf = new WWW::Curl::Form
+        This method constructs a new WWW::Curl::Form object.
+
+    $curlf->formadd(FIELDNAME, VALUE)
+        This method adds a field with a given value, to the form that is being submitted.
+
+    $curlf->formaddfile(FILENAME, DESCRIPTION, TYPE)
+        This method will add a file to the form. The description is the name of the field
+        that you form expects the data to be submitted in.
 
 =head1 COMPATIBILITY
 
@@ -216,12 +256,16 @@ Not implemented and won't be as this method is considered deprecated.
 
 =item curl_formadd
 
-Not yet implemented.
+Seems to be working.
+
+=item curl_formaddfile
+
+Seems to be working.
 
 =item curl_formfree
 
-When WWW::Curl::Form support is added, this function will be used internally,
-but won't be accessible from the public API.
+Used internally. Not exposed through the public API, as this call has no relevance
+to Perl code.
 
 =item curl_free
 
@@ -245,6 +289,10 @@ Only used internally, not exposed through the public API.
 
 Not implemented.
 
+=item curl_global_cleanup
+
+Only used internally and called automatically upon exit.
+
 =item curl_slist_append
 
 Only used internally, not exposed through the public API.
@@ -257,6 +305,10 @@ Only used internally, not exposed through the public API.
 
 Not implemented and won't be, as this method is considered deprecated.
 
+=item curl_version
+
+Seems to work.
+
 =item curl_version_info
 
 Not yet implemented.
@@ -266,22 +318,64 @@ Not yet implemented.
 Most methods are either not exposed through the WWW::Curl::Multi API or they behave differently
 than it's C counterpart. Please see the section about WWW::Curl::Multi above.
 
+=item curl_multi_fdset
+
+This method returns three arrayrefs: the read, write and exception fds libcurl knows about.
+In the case of no file descriptors in the given set, an empty array is returned.
+
+=item constant
+
+Useless.
+
+=back
+
+=head1 NUANCES
+
+=head2 Header output for redirects
+
+It might be surprising that if C<CURLOPT_FOLLOWLOCATION> is set and header output was enabled, headers show up for all http responses.
+The reasoning behind that and possible code adjustments are outlined here: L<https://rt.cpan.org/Ticket/Display.html?id=61569>.
+
+=head2 CURLOPT_PRIVATE
+
+Despite what the libcurl manual says, in Perl land, only string values are suitable for this option.
+
+=head1 ADDITIONAL METHODS
+
+=head2 On WWW::Curl::Easy objects
+
+=over 
+
+=item pushopt
+
+Like C<setopt> but instead of overriding any previously set values it adds it 
+to the end. Can be used with C<CURLOPT_HTTPHEADER>, C<CURLOPT_QUOTE> and 
+C<CURLOPT_POSTQUOTE>.
+
+
 =back
 
 =head1 USAGE CASES
 
-The standard Perl WWW module, LWP should be used in most cases to work with
-the HTTP or FTP protocol from Perl. However, there are some cases where LWP doesn't
-perform well. One is speed and the other is paralellism. WWW::Curl is much faster,
-uses much less CPU cycles and it's capable of non-blocking parallel requests.
+WWW::Curl is a thin binding on top of libcurl, to make using libcurl possible from Perl land.
+Because of this, the module is less like Perl and more like C in coding style.
+
+There is a new module, L<http://search.cpan.org/perldoc?WWW::Curl::Simple>,
+which wraps this module into a more Perlish and userfriendly package.
+
+The standard Perl WWW module, LWP should probably be used in most cases to work with HTTP or FTP from Perl.
+However, there are some cases where LWP doesn't perform well. One is speed and the other is parallelism. 
+WWW::Curl is much faster, uses much less CPU cycles and it's capable of non-blocking parallel requests.
 
 In some cases, for example when building a web crawler, cpu usage and parallel downloads are
 important considerations. It can be desirable to use WWW::Curl to do the heavy-lifting of
 a large number of downloads and wrap the resulting data into a Perl-friendly structure by
-HTTP::Response.
+HTTP::Response or use WWW::Curl::Simple to do that for you.
 
 =head1 CHANGES
 
+Version 4.01 - 4.07 adds several bugfixes and extends functionality coverage. See Changes file.
+
 Version 4.00 added new documentation, the build system changed to Module::Install,
 the test suite was rewritten to use Test::More, a new calling syntax for WWW::Curl::Multi
 was added, memory leak and other bugfixes added, Perl 5.6 and libcurl 7.10.8 as minimum
@@ -299,7 +393,7 @@ Version 3.02 adds some backwards compatibility for scripts still using
 
 Version 3.01 added some support for pre-multi versions of libcurl.
 
-Version 3.00 adds WWW::Curl::Multi interface, and a new module names
+Version 3.00 adds WWW::Curl::Multi interface, and new module names
 following perl conventions (WWW::Curl::Easy rather than WWW::Curl::easy),
 by Sebastian Riedel <sri at cpan.org>.
 
@@ -312,7 +406,8 @@ reworked from the previous Curl::easy release (1.21) by Cris Bailiff.
 
 =head1 AUTHORS
 
-Currently maintained by Cris Bailiff <c.bailiff+curl at devsecure.com>
+Currently maintained by Cris Bailiff <c.bailiff+curl at devsecure.com> and
+Balint Szilakszi <szbalint at cpan.org>.
 
 Original Author Georg Horn <horn@koblenz-net.de>, with additional callback,
 pod and test work by Cris Bailiff <c.bailiff+curl@devsecure.com> and
@@ -322,9 +417,9 @@ repackaged the module into a more modern form.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2000-2005,2008 Daniel Stenberg, Cris Bailiff,
+Copyright (C) 2000-2005,2008-2010 Daniel Stenberg, Cris Bailiff,
 Sebastian Riedel, Balint Szilakszi et al.
- 
+
 You may opt to use, copy, modify, merge, publish, distribute and/or sell
 copies of the Software, and permit persons to whom the Software is furnished
 to do so, under the terms of the MPL or the MIT/X-derivate licenses. You may
@@ -332,4 +427,10 @@ pick one of these licenses.
 
 =head1 SEE ALSO
 
-http://curl.haxx.se
+L<http://curl.haxx.se>
+
+L<http://search.cpan.org/perldoc?WWW::Curl::Simple>
+
+L<libcurl(3)>
+
+The development source code is also available: L<http://github.com/szbalint/WWW--Curl/tree/master>
